/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.request.impl;

import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IRequestor;
import org.netkernel.request.IResponse;
import org.netkernel.scheduler.TraceDebugCapture;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IResolution;
import org.netkernel.util.ExclusionSet;

/** Factory for creating requests
 * @author Tony Butterfield
 */
public class RequestFactory
{
	public static String toString(IRequest aRequest)
	{	
		StringBuilder sb= new StringBuilder(256);
		appendToStringInner(aRequest,sb,true);
		return sb.toString();
	}
	
	public static String toShortString(IRequest aRequest)
	{	
		StringBuilder sb= new StringBuilder(256);
		appendToStringInner(aRequest,sb,false);
		return sb.toString();
	}
	
	public static String priorityString(int aPriority)
	{	String result;
		switch(aPriority)
		{	case IRequest.PRIORITY_MIN: result="MIN"; break;
			case IRequest.PRIORITY_STD: result="STD"; break;
			case IRequest.PRIORITY_MAX: result="MAX"; break;
			default: result="??"; break;
		}
		return result;
	}
	
	public static String verbString(int aVerb)
	{
		String verb;
		switch(aVerb)
		{	case IRequest.VERB_SOURCE: verb="SOURCE"; break;
			case IRequest.VERB_SINK: verb="SINK"; break;
			case IRequest.VERB_DELETE: verb="DELETE"; break;
			case IRequest.VERB_EXISTS: verb="EXISTS"; break;
			case IRequest.VERB_NEW: verb="NEW"; break;
			case IRequest.VERB_TRANSREPT: verb="TRANSREPT"; break;
			case IRequest.VERB_RESOLVE: verb="RESOLVE"; break;
			case IRequest.VERB_META: verb="META"; break;
			default: verb="??"; break;
		};
		return verb;
	}
	
	public static int parseVerb(String aVerb)
	{	int result;
		if (aVerb.equals("SOURCE"))
		{	result=IRequest.VERB_SOURCE;
		}
		else if (aVerb.equals("SINK"))
		{	result=IRequest.VERB_SINK;
		}
		else if (aVerb.equals("DELETE"))
		{	result=IRequest.VERB_DELETE;
		}
		else if (aVerb.equals("EXISTS"))
		{	result=IRequest.VERB_EXISTS;
		}
		else if (aVerb.equals("NEW"))
		{	result=IRequest.VERB_NEW;
		}
		else if (aVerb.equals("TRANSREPT"))
		{	result=IRequest.VERB_TRANSREPT;
		}
		else if (aVerb.equals("RESOLVE"))
		{	result=IRequest.VERB_RESOLVE;
		}
		else if (aVerb.equals("META"))
		{	result=IRequest.VERB_META;
		}
		else if (aVerb.equals("ALL"))
		{	result=IRequest.VERB_ALL;
		}
		else
		{	result=0;
		}
		return result;
	}
	
	private static void appendToStringInner(IRequest aRequest, StringBuilder sb, boolean aLong)
	{
		sb.append(verbString(aRequest.getVerb()));
		sb.append(' ');
		if (aRequest.getVerb()==IRequest.VERB_TRANSREPT)
		{	sb.append("from ");
			IResponse response = (IResponse)aRequest.getValue(IRequest.TYPE_SOURCED_RESPONSE);
			if (response!=null && response.getRepresentation()!=null)
			{	sb.append(aLong?response.getRepresentation().getClass().getName():getShortClassname(response.getRepresentation().getClass()));
			}
			else
			{	sb.append("null");
			}
			sb.append(" to ");
			sb.append(aLong?aRequest.getRepresentationClass().getName():getShortClassname(aRequest.getRepresentationClass()));
			if (aLong)
			{	sb.append(" in ");
				sb.append(aRequest.getRequestScope());
			}
		}
		else if (aRequest.getVerb()==IRequest.VERB_RESOLVE)
		{	IRequest toResolve = (IRequest)aRequest.getValue(IRequest.TYPE_RESOLVE_REQUEST);
			if (toResolve!=null)
			{	sb.append('[');
				appendToStringInner(toResolve,sb,aLong);
				sb.append(" ]");
			}
		}
		else
		{	String id=aRequest.getIdentifier().toString();
			if (id.length()==0)
			{	id="\"\"";
			}
			sb.append(id);
			if (aRequest.getVerb()==IRequest.VERB_SOURCE)
			{	sb.append(" as ");
				sb.append(aLong?aRequest.getRepresentationClass().getName():getShortClassname(aRequest.getRepresentationClass()));
			}
			
			if (aLong)
			{	sb.append(" in ");
				sb.append(aRequest.getRequestScope());
			}
		}		
	}
	
	private static String getShortClassname(Class aClass)
	{
		String name=aClass.getName();
		int i=name.lastIndexOf('.');
		if (i>0)
		{	name=name.substring(i+1);
		}
		return name;
	}
	
	public static IRequest cloneWithScope(IRequest aOther, IRequestScopeLevel aScope)
	{	final IRequestScopeLevel scope=aScope;
		return new RequestCloneImpl(aOther)
		{	public IRequestScopeLevel getRequestScope()
			{	return scope;
			}
		};
	}

	public static IRequest cloneWithRequestor(IRequest aOther, IRequestor aRequestor)
	{	final IRequestor requestor=aRequestor;
		return new RequestCloneImpl(aOther)
		{	public IRequestor getRequestor()
			{	return requestor;
			}
		};
	}
	
	public static IRequest cloneWithScopeAndRequestor(IRequest aOther, IRequestScopeLevel aScope, IRequestor aRequestor, IIdentifier aEndpointIdentifier)
	{	final IRequestScopeLevel scope=aScope;
		final IRequestor requestor=aRequestor;
		final Object[] values = new Object[] {aOther.getValue(0),aEndpointIdentifier};
		return new RequestCloneImpl(aOther)
		{	public IRequestScopeLevel getRequestScope()
			{	return scope;
			}
			public IRequestor getRequestor()
			{	return requestor;
			}
			public Object[] getValues()
			{	return values;
			}
			public Object getValue(int aType)
			{	Object result=null;
				if (aType>=0 && aType<values.length)
				{	result=values[aType];
				}
				return result;
			}			
		};
	}
	
	/*
	public static IRequest createKernelIssuedRequest(IRequest aOther, IRequestScopeLevel aScope, IRequestor aRequestor, IIdentifier aEndpointIdentifier, IRequestResponseFields aStickyHeaders)
	{
		final Object[] values = new Object[] {aOther.getValue(0),aEndpointIdentifier,aStickyHeaders};
		return new RequestImpl(aOther.getVerb(),aOther.getIdentifier()aScope,aOther.getRepresentationClass(),aOther.getParent(),aRequestor,aOther.getHeaders()
	}
	*/
	
	
	
	public static IRequest cloneWithRepresentationClass(IRequest aOther, Class aRepresentationClass)
	{	final Class representationClass=aRepresentationClass;
		return new RequestCloneImpl(aOther)
		{	public Class getRepresentationClass()
			{	return representationClass;
			}
		};
	}

	public static IRequest cloneWithVerb(IRequest aOther, int aVerb)
	{	final int verb=aVerb;
		return new RequestCloneImpl(aOther)
		{	public int getVerb()
			{	return verb;
			}
		};
	}
	
	public static IRequest cloneAsRootRequest(IRequest aOther)
	{	return new RequestCloneImpl(aOther)
		{	public IRequest getParent()
			{	return null;
			}
		};
	}
	
	/** Create a new resolve request
	 * @param aToResolve the request to resolve
	 * @param aResolveExcludedSpaces an exclusion set to hold eliminated spaces, if null one will be created (size 16)
	 * @param aDebug optional list to accumulate debug trace from request resolution
	 * @param aParent the parent request
	 * @param aRequestor the requestor
	 * @return
	 */
	public static IRequest createResolveRequest(IRequest aToResolve, ExclusionSet aResolveExcludedSpaces, TraceDebugCapture aDebug, IRequest aParent, IRequestor aRequestor)
	{	if (aResolveExcludedSpaces==null)
		{	aResolveExcludedSpaces = new ExclusionSet(16);
		}
		Object[] values = new Object[] { aToResolve, aResolveExcludedSpaces, aDebug };
		IRequestResponseFields headers=RequestResponseFieldsImpl.EMPTY;
		return new RequestImpl(IRequest.VERB_RESOLVE,aToResolve.getIdentifier(),aToResolve.getRequestScope(),IResolution.class,aParent, aRequestor, headers, values);
	}
	
	/**
	 * @param aRequestIn
	 * @param aSourcedResponse
	 * @param aRequestor
	 * @return
	 */
	public static IRequest createTransreptRequest(IRequest aRequestIn, IResponse aSourcedResponse, IRequestor aRequestor)
	{
		Object[] values = new Object[] { aSourcedResponse };
		IRequestResponseFields headers=RequestResponseFieldsImpl.EMPTY;
		return new RequestImpl(IRequest.VERB_TRANSREPT,aRequestIn.getIdentifier(),aRequestIn.getRequestScope(),aRequestIn.getRepresentationClass(),aRequestIn,aRequestor,headers,values);
	}
	
}