/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.request.impl;

import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IRequestor;
import org.netkernel.urii.IIdentifier;

/** Implementation of IRequest
 * @author  tab
 */
public class RequestImpl implements IRequest
{
	private final int mVerb;
	private final IIdentifier mIdentifier;
	private final Class mRepresentationClass;
	private final IRequestScopeLevel mScope;
	private final IRequest mParent;
	private final IRequestor mRequestor;
	private final IRequestResponseFields mHeaders;
	private final Object[] mValues;
	
	/** Creates a new instance of RequestImpl */
	public RequestImpl(int aVerb, IIdentifier aIdentifier, IRequestScopeLevel aScope, Class aRepresentationClass, IRequest aParent, IRequestor aRequestor, IRequestResponseFields aHeaders, Object[] aValues)
	{	mVerb=aVerb;
		mIdentifier=aIdentifier;
		mScope=aScope;
		mRepresentationClass=aRepresentationClass;
		mParent=aParent;
		mRequestor=aRequestor;
		mHeaders=aHeaders;
		mValues=aValues;
	}
	
	public Class getRepresentationClass()
	{	return mRepresentationClass;
	}
	
	public IRequestScopeLevel getRequestScope()
	{	return mScope;
	}
	
	public int getPriority()
	{	int result;
		Integer priority=(Integer)getHeaders().getValue(IRequest.HEADER_PRIORITY);
		if (priority==null)
		{	result=IRequest.PRIORITY_STD;
		}
		else
		{	result=priority.intValue();
		}
		return result;
	}
	
	public IIdentifier getIdentifier()
	{	return mIdentifier;
	}
	
	public int getVerb()
	{	return mVerb;
	}
	
	public String toString()
	{	return RequestFactory.toString(this);
	}
	
	public IRequest getParent()
	{	return mParent;
	}

	public IRequestor getRequestor()
	{	return mRequestor;
	}

	public IRequestResponseFields getHeaders()
	{	return mHeaders;
	}

	public Object getValue(int aType)
	{	Object result=null;
		if (mValues!=null && aType>=0 && aType<mValues.length)
		{	result=mValues[aType];
		}
		return result;
	}

	public Object[] getValues()
	{	return mValues;
	}
}	