/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.request.impl;

import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.urii.ISpace;

/** Implementation of IRequestScopeLevel
 * @author tab
 */
public class RequestScopeLevelImpl implements IRequestScopeLevel
{
	private final ISpace mSpace;
	private final IRequestScopeLevel mParent;
	private final int mDepth;
	private final int mAvailableDepth;

	/** Creates a new instance of RequestSpaceImpl */
	private RequestScopeLevelImpl(ISpace aSpace, IRequestScopeLevel aParent, int aDepth, boolean aScopeDurable, int aAvailableDepth)
	{
		mSpace = aSpace;
		mParent = aParent;
		//Depth and Durability are stored together to reduce memory overhead and are infrequently accessed
		// don't worry about the special case of zero, all depths must be at least one
		mAvailableDepth = aScopeDurable?-aAvailableDepth:aAvailableDepth;
		mDepth=aDepth;
	}

	public ISpace getSpace()
	{	return mSpace;
	}

	public IRequestScopeLevel getParent()
	{	return mParent;
	}

	public int getDepth()
	{	return mDepth;
	}
	
	public int getAvailableDepth()
	{	return Math.abs(mAvailableDepth);
	}

	public String toString()
	{	StringBuilder sb=new StringBuilder();
		IRequestScopeLevel rc = this;
		do
		{	sb.append(rc.getSpace().toString());
			rc = rc.getParent();
			if (rc != null)
			{	sb.append("... ");
			}
		} while (rc != null);
		return sb.toString();
	}

	public boolean isDurable()
	{	return mAvailableDepth<0;
	}
	
	/** Factory method to create a scope level  to wrap around an existing spacet
	 * @param aSpace The space we want to put into the scope.
	 * @param aParent The parent scope
	 */
	public static IRequestScopeLevel createNonDurableScopeLevel(ISpace aSpace,IRequestScopeLevel aParent)
	{	return new RequestScopeLevelImpl(aSpace, aParent, aParent.getDepth() + 1,false, aParent.getAvailableDepth()+1);
	}
	
	/** Factory method to create a durable request scope, if need, to wrap around an existing space
	 * @param aSpace The space we want to put into the request scope
	 * @param aParent The parent scope
	 */
	public static IRequestScopeLevel createDurableScopeLevel(ISpace aSpace,IRequestScopeLevel aParent)
	{	return createScopeLevel(aSpace,aParent,true);
	}	

	public static IRequestScopeLevel createScopeLevel(ISpace aSpace,IRequestScopeLevel aParent, boolean aDurable)
	{	return new RequestScopeLevelImpl(aSpace, aParent, aParent.getDepth()+1, aDurable,aParent.getAvailableDepth()+1);
	}	

	/** Factory method to create a root request scope with no parent
	 * @param aSpace
	 */
	public static IRequestScopeLevel createRootScopeLevel(ISpace aSpace)
	{	return new RequestScopeLevelImpl(aSpace, null, 1,false,1);
	}
	
	public static IRequestScopeLevel createOrphanedRootScopeLevel(ISpace aSpace, IRequestScopeLevel aParent)
	{	return new RequestScopeLevelImpl(aSpace, null, aParent.getDepth()+1 ,false, 1);
	}
	public static IRequestScopeLevel createOrphanedRootScopeLevel(ISpace aSpace, int aDepthOfOrphan)
	{	return new RequestScopeLevelImpl(aSpace, null, aDepthOfOrphan ,false, 1);
	}
	
	public static IRequestScopeLevel insertRootScopeLevel(ISpace aSpace, IRequestScopeLevel aHead)
	{	IRequestScopeLevel root = createRootScopeLevel(aSpace);
		return recurseWrapRoot(null,root,aHead);
	}
	
	public static IRequestScopeLevel createScopeLevelWithout(ISpace aSpace, IRequestScopeLevel aParent)
	{	IRequestScopeLevel result=aParent;
		if (result.getSpace()==aSpace)
		{	result=result.getParent();
		}
		return result;
	}
	
	private static IRequestScopeLevel recurseWrapRoot(IRequestScopeLevel aOldRoot, IRequestScopeLevel aNewRoot, IRequestScopeLevel aHead)
	{	IRequestScopeLevel originalParent=aHead.getParent();
		IRequestScopeLevel newParent;
		if (originalParent!=aOldRoot)
		{	newParent=recurseWrapRoot(aOldRoot,aNewRoot,originalParent);
		}
		else
		{	newParent=aNewRoot;
		}
		return new RequestScopeLevelImpl(aHead.getSpace(),newParent,aHead.getDepth()+1,aHead.isDurable(),aHead.getAvailableDepth()+1);
	}
	
	public static IRequestScopeLevel appendScopeLevel(IRequestScopeLevel aToAppend, IRequestScopeLevel aParent)
	{	return createScopeLevel(aToAppend.getSpace(),aParent,aToAppend.isDurable());
	}
}