/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.scheduler;

import org.netkernel.request.IExpiry;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IResponseMeta;
import org.netkernel.request.impl.MetaImpl;

public abstract class IInternalResponseMeta extends MetaImpl
{
	public abstract int getUnresolvedScope();
	public abstract int getResolvedScope();
	public abstract int getLocalCost();
	public abstract int getTotalCost();
	public abstract IInternalResponseMeta[] getCostDeterminants();
	public abstract int getCostDeterminantsLength();
	public abstract IExpiry[] getExpiryDeterminants();
	
	public static IInternalResponseMeta[] NONE=new IInternalResponseMeta[0];

	public IInternalResponseMeta(IExpiry aExpiry, IRequestResponseFields aUserMetaData)
	{	super(aExpiry,aUserMetaData);
	}
	
	public String toString()
	{	return "IInternalResponseMeta( cost="+getTotalCost()+" unresolved-scope="+getUnresolvedScope()+" resolved-scope="+getResolvedScope()+" expiry="+getExpiry().toString()+"("+getExpiry().isExpired(System.currentTimeMillis())+") )";
	}

	public static IInternalResponseMeta create(IExpiry aExpiry, IRequestResponseFields aUserMeta, int aUnresolvedScope, int aResolvedScope, int aLocalCost, int aTotalCost, IInternalResponseMeta[] aCostDeterminants, int aCostDeterminantsLength, IExpiry[] aExpiryDeterminants)
	{	return new InternalResponseMeta(aExpiry,aUserMeta,aUnresolvedScope, aResolvedScope, aLocalCost, aTotalCost, aCostDeterminants, aCostDeterminantsLength, aExpiryDeterminants);
	}
	
	public static IInternalResponseMeta cloneWithScope(final IInternalResponseMeta aOther, final int aUnresolvedScope)
	{	return new IInternalResponseMeta(aOther.getExpiry(),aOther.getUserMetaData())
		{
			public int getUnresolvedScope()
			{	return aUnresolvedScope;
			}

			public int getResolvedScope()
			{	return aUnresolvedScope;
			}

			public int getLocalCost()
			{	return aOther.getLocalCost();
			}

			public int getTotalCost()
			{	return aOther.getTotalCost();
			}

			public IExpiry getExpiry()
			{	return aOther.getExpiry();
			}

			public IInternalResponseMeta[] getCostDeterminants()
			{	return aOther.getCostDeterminants();
			}

			public IRequestResponseFields getUserMetaData()
			{	return aOther.getUserMetaData();
			}

			public IExpiry[] getExpiryDeterminants()
			{	return aOther.getExpiryDeterminants();
			}	
			
			public int getCostDeterminantsLength()
			{	return aOther.getCostDeterminantsLength();
			}
		};
	}
}